#!/bin/bash
# mongoowl snmp agent install script.

INST_TITLE="snmp agent for mongoowl"

SNMP_SCRIPT_PATH="/var/lib/iostatcapture"
SNMP_CONF_FILE="/etc/snmp/snmpd.conf"

RETURN_ERROR=0
RETURN_SUCCESS=0

STR_BAR01="======================================================"
STR_BAR02="------------------------------------------------------"

# pause until [Enter] key is pressed
function pause(){
	read -p "Press [Enter] key to continue..."
	echo "pause"
}

function fn_msg_title_print(){
	local msg="$*"
	
	echo ""
	echo "${STR_BAR01}"
	echo "- ${msg}"
	echo "${STR_BAR02}"
}

function fn_msg_header_print(){
	local msg="$*"
	
	echo ""
	echo "- ${msg}"
}

function fn_check_snmp_installed(){
	fn_msg_header_print "Checking if the snmp service is installed..."
	
	local INSTALLED=`rpm -qa | grep net-snmp`
	
	if [ "$INSTALLED" == "" ]; then
		echo "The NET-SNMP 5.5 is not installed. You have to install NET-SNMP 5.5 for using the mongoowl's snmp functionality."
		echo "To install the snmp service , Use yum command as follow:"
		echo "# yum install net-snmp net-snmp-utils" 

		exit ${RETURN_ERROR}
	else
		INSTALLED=`rpm -qa | grep net-snmp-utils`
		
		if [ "$INSTALLED" == "" ]; then
			echo "The NET-SNMP 5.5 is not installed. You have to install NET-SNMP 5.5 for using the mongoowl's snmp functionality."
			echo "To install the snmp service , Use yum command as follow:"
			echo "# yum install net-snmp net-snmp-utils" 
	
			exit ${RETURN_ERROR}
		else
			
			INSTALLED=`snmpd --version | grep version | grep 5.5`
			
			if [ "$INSTALLED" == "" ]; then
				echo "The NET-SNMP version is not 5.5. You have to install NET-SNMP 5.5 for using the mongoowl's snmp functionality."
			else
				echo "The snmp service is installed. OK!"
			fi
		fi
	fi
	 
}

function fn_check_iptables_snmp_port_opened(){
	fn_msg_header_print "Checking if the snmp port is opened..."
	
	local OPEND=`cat /etc/sysconfig/iptables | grep udp | grep 161`
	
	if [ "$OPEND" == "" ]; then
		echo "The snmp port is not opened. You have to open the snmp port for using the mongoowl's snmp functionality."
		echo "To open the snmp port , Edit /etc/sysconfig/iptables and add a line as follow:"
		echo ""
		echo " -A INPUT -m state --state NEW -m udp -p udp --dport 161 -j ACCEPT"
		echo ""
		echo "And restart iptables service as follow:"
		echo "# service iptables restart"
		echo ""
		echo "Press [Ctrl+C] key to Stop this installation or"
		pause
	else
		echo "The snmp port is opened. OK!"
	fi
}

function fn_check_sysstat_installed(){
	fn_msg_header_print "Checking if the sysstat package is installed..."
	
	local INSTALLED=`rpm -qa | grep sysstat`
	if [ "$INSTALLED" == "" ]; then
		echo "The sysstat package is not installed. You have to install the sysstat package for using the mongoowl's iostat monitoring."
		echo "To install the sysstat package , Use yum command as follow:"
		echo "# yum install sysstat" 

		exit ${RETURN_ERROR}
	else
		echo "The sysstat package is installed. OK!"
	fi
}

function fn_check_selinux_disabled(){
	fn_msg_header_print "Checking if SELinux is disabled..."
	
	local ENABLED=`cat /selinux/enforce`
	local disable=""
	
	if [ "$ENABLED" == 1 ]; then
		echo "SELinux is enabled. You have to disable SELinux for using the mongoowl's snmp functionality. disable? (yes/no):"
		read disable
		if [ $disable == "yes" ]; then			
			setenforce 0
			echo "Disabled selinux. This is temporary. You can modify the selinux's value with \"disabled\" in /etc/selinux/config file to disable SELinux permanently."
			pause
		else
			exit ${RETURN_ERROR}
		fi
	else
		echo "SELinux is disabled. OK!"
	fi
}

function fn_add_iostat_account(){
	fn_msg_header_print "Adding an account for snmp agent..."
	
	mkdir ${SNMP_SCRIPT_PATH}
	useradd -r -d ${SNMP_SCRIPT_PATH} -s /bin/false iostat
	touch ${SNMP_SCRIPT_PATH}/stats
	chown -R iostat ${SNMP_SCRIPT_PATH}
	
	echo "The account is Added. OK!"
}

function fn_install_run_script(){
	fn_msg_header_print "Installing the shell-script files to ${SNMP_SCRIPT_PATH}/..."
	
	cp ./iostatcapture_run ${SNMP_SCRIPT_PATH}/iostatcapture
	cp ./topForMongod ${SNMP_SCRIPT_PATH}
	cp ./topForMongos ${SNMP_SCRIPT_PATH}
	chmod 755 ${SNMP_SCRIPT_PATH}/*
	chown -R iostat ${SNMP_SCRIPT_PATH}
	
	echo "The shell-script files is installed. OK!"
}

function fn_install_service_script(){
	fn_msg_header_print "Installing servcie script file to /etc/init.d/iostatcapture for iostatcapture..."
	
	cp ./iostatcapture_service /etc/init.d/iostatcapture
	chmod 755 /etc/init.d/iostatcapture
	chkconfig iostatcapture on
	
	echo "The servcie script file is installed. OK!"
}

function fn_service_iostatcapture_start(){
	fn_msg_header_print "String iostatcapture service..."
	
	service iostatcapture stop
	service iostatcapture start
}

function fn_configure_snmp(){
	fn_msg_header_print "Configuring the ${SNMP_CONF_FILE} file..."
	
	local CONFIGURED=`cat ${SNMP_CONF_FILE} | grep mongoowl`
	local CURDATE=`date +"%y%m%d_%H%M%S"`
	
	if [ "$CONFIGURED" == "" ]; then	
		cp ${SNMP_CONF_FILE} ${SNMP_CONF_FILE}.${CURDATE}
		echo "Copied the original ${SNMP_CONF_FILE} to the ${SNMP_CONF_FILE}.${CURDATE}."
		cat ${SNMP_CONF_FILE} | grep -v ^access > ${SNMP_CONF_FILE}.tmp
		cat >>${SNMP_CONF_FILE}.tmp<<EOF

###############################################################################
# for mongoowl:
view    all           included   .1                  80
access   notConfigGroup ""      any       noauth    exact  all        none none
extend .1.3.6.1.4.1.2021.52 iostat /bin/cat ${SNMP_SCRIPT_PATH}/stats
extend .1.3.6.1.4.1.2021.53 topForMongod /bin/sh ${SNMP_SCRIPT_PATH}/topForMongod
extend .1.3.6.1.4.1.2021.54 topForMongos /bin/sh ${SNMP_SCRIPT_PATH}/topForMongos
EOF

		mv -f ${SNMP_CONF_FILE}.tmp ${SNMP_CONF_FILE}
		
		echo "The ${SNMP_CONF_FILE} file is configured. OK!"
	else
		echo "Already configured the ${SNMP_CONF_FILE} file."
		pause
	fi
}

function fn_service_snmpd_restart(){
	fn_msg_header_print "Restarting snmpd service..."
	
	service snmpd restart
}

function fn_check_if_snmpinst_success(){
	fn_msg_header_print "Checking if ${INST_TITLE} work well..."
	echo "To check if ${INST_TITLE} work well, Use snmpwork command as follow:"
	echo "# snmpwalk -v2c -c public localhost .1.3.6.1.4.1.2021.53"
}

function fn_main(){
	fn_msg_title_print "Installing ${INST_TITLE}..."
	
	fn_check_snmp_installed
	fn_check_iptables_snmp_port_opened
	fn_check_sysstat_installed
	fn_check_selinux_disabled
	fn_add_iostat_account
	fn_install_run_script
	fn_install_service_script
	fn_service_iostatcapture_start
	fn_configure_snmp
	fn_service_snmpd_restart
	fn_check_if_snmpinst_success
}

### main function call
fn_main


